#include "td-tut-2-main.h"

//# Enemies

#define ENEMY_MAX_COUNT 400
#define ENEMY_TYPE_NONE 0
#define ENEMY_TYPE_MINION 1

typedef struct Enemy
{
  int16_t currentX, currentY;
  uint8_t enemyType;
} Enemy;

Enemy enemies[ENEMY_MAX_COUNT];
int enemyCount = 0;

void EnemyInit()
{
  for (int i = 0; i < ENEMY_MAX_COUNT; i++)
  {
    enemies[i] = (Enemy){0};
  }
  enemyCount = 0;
}

void EnemyDraw()
{
  for (int i = 0; i < enemyCount; i++)
  {
    Enemy enemy = enemies[i];
    switch (enemy.enemyType)
    {
    case ENEMY_TYPE_MINION:
      DrawCube((Vector3){enemy.currentX, 0.2f, enemy.currentY}, 0.4f, 0.4f, 0.4f, GREEN);
      break;
    }
  }
}

Enemy *EnemyTryAdd(uint8_t enemyType, int16_t currentX, int16_t currentY)
{
  if (enemyCount >= ENEMY_MAX_COUNT)
  {
    return 0;
  }

  Enemy *enemy = &enemies[enemyCount++];
  enemy->currentX = currentX;
  enemy->currentY = currentY;
  enemy->enemyType = enemyType;
  return enemy;
}

//# Towers

#define TOWER_MAX_COUNT 400
#define TOWER_TYPE_NONE 0
#define TOWER_TYPE_BASE 1
#define TOWER_TYPE_GUN 2

typedef struct Tower
{
  int16_t x, y;
  uint8_t towerType;
} Tower;

Tower towers[TOWER_MAX_COUNT];
int towerCount = 0;

void TowerInit()
{
  for (int i = 0; i < TOWER_MAX_COUNT; i++)
  {
    towers[i] = (Tower){0};
  }
  towerCount = 0;
}

Tower *TowerGetAt(int16_t x, int16_t y)
{
  for (int i = 0; i < towerCount; i++)
  {
    if (towers[i].x == x && towers[i].y == y)
    {
      return &towers[i];
    }
  }
  return 0;
}

Tower *TowerTryAdd(uint8_t towerType, int16_t x, int16_t y)
{
  if (towerCount >= TOWER_MAX_COUNT)
  {
    return 0;
  }

  Tower *tower = TowerGetAt(x, y);
  if (tower)
  {
    return 0;
  }

  tower = &towers[towerCount++];
  tower->x = x;
  tower->y = y;
  tower->towerType = towerType;
  return tower;
}

void TowerDraw()
{
  for (int i = 0; i < towerCount; i++)
  {
    Tower tower = towers[i];
    DrawCube((Vector3){tower.x, 0.125f, tower.y}, 1.0f, 0.25f, 1.0f, GRAY);
    switch (tower.towerType)
    {
    case TOWER_TYPE_BASE:
      DrawCube((Vector3){tower.x, 0.4f, tower.y}, 0.8f, 0.8f, 0.8f, MAROON);
      break;
    case TOWER_TYPE_GUN:
      DrawCube((Vector3){tower.x, 0.2f, tower.y}, 0.8f, 0.4f, 0.8f, DARKPURPLE);
      break;
    }
  }
}

//# Game

void InitGame()
{
  TowerInit();
  EnemyInit();

  TowerTryAdd(TOWER_TYPE_BASE, 0, 0);
  TowerTryAdd(TOWER_TYPE_GUN, 2, 0);
  EnemyTryAdd(ENEMY_TYPE_MINION, 5, 4);
}

int main(void)
{
  int screenWidth, screenHeight;
  GetPreferredSize(&screenWidth, &screenHeight);
  InitWindow(screenWidth, screenHeight, "Tower defense");
  SetTargetFPS(30);

  Camera3D camera = {0};
  camera.position = (Vector3){0.0f, 10.0f, 5.0f};
  camera.target = (Vector3){0.0f, 0.0f, 0.0f};
  camera.up = (Vector3){0.0f, 0.0f, -1.0f};
  camera.fovy = 45.0f;
  camera.projection = CAMERA_PERSPECTIVE;

  InitGame();

  while (!WindowShouldClose())
  {
    if (IsPaused()) {
      // canvas is not visible in browser - do nothing
      continue;
    }
    BeginDrawing();
    ClearBackground(DARKBLUE);

    BeginMode3D(camera);
    DrawGrid(10, 1.0f);
    TowerDraw();
    EnemyDraw();
    EndMode3D();

    DrawText("Tower defense tutorial", 5, 5, 20, WHITE);
    EndDrawing();
  }

  CloseWindow();

  return 0;
}